package cn.com.qmth.stmms.biz.mark.service.Impl;

import cn.com.qmth.stmms.biz.exam.model.*;
import cn.com.qmth.stmms.biz.exam.service.*;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.mark.model.*;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.*;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import cn.com.qmth.stmms.common.enums.MarkStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * 评卷任务服务实现
 *
 * @author LS
 */
@Service("taskService")
public class TaskServiceImpl implements TaskService {

    private static Logger log = LoggerFactory.getLogger(TaskServiceImpl.class);

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkTrackService trackService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private MarkSpecialTagService markSpecialTagService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private TrialService trialService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private UserService userService;

    @Autowired
    private FileService fileService;

    @Override
    public List<Task> findByQuery(MarkLibrarySearchQuery query) {
        List<Task> list = new LinkedList<Task>();
        query = libraryService.findByQuery(query);
        if (query.getCurrentCount() > 0) {
            for (MarkLibrary library : query.getResult()) {
                list.add(build(library));
            }
        }
        return list;
    }

    @Override
    public Task build(ArbitrateHistory history, MarkGroup group) {
        ExamSubject subject = subjectService.find(group.getExamId(), group.getSubjectCode());
        ExamStudent student = studentService.findByExamIdAndExamNumber(history.getExamId(), history.getExamNumber());
        List<MarkLibrary> libraryList = libraryService.findByStudentAndGroup(student.getId(), group.getNumber());
        Task task = new Task();
        task.setExist(true);
        task.setStudentId(history.getStudentId());
        task.setLibraryId(history.getId());
        task.setStudentNumber(history.getSecretNumber());
        task.setMarkStepList(buildMarkStep(group, null));
        task.setPictureUrls(fileService.getSliceUris(student.getExamId(), student.getSecretNumber(), 1,
                student.getSliceCount()));
        task.setPictureConfig(group.getPictureConfigList());
        task.setJsonUrl(fileService.getJsonUri(student.getExamId(), student.getSecretNumber()));
        if (subject.getPaperFileType() != null) {
            task.setPaperUrl(fileService.getPaperUri(subject.getExamId(), subject.getCode(), subject.getPaperFileType()));
        }
        if (subject.getAnswerFileType() != null) {
            task.setAnswerUrl(fileService.getAnswerUri(subject.getExamId(), subject.getCode(),
                    subject.getAnswerFileType()));
        }
        task.setObjectiveScore(student.getObjectiveScore() != null ? student.getObjectiveScore() : 0);
        task.setMarkTime(history.getUpdateTime());
        // 构造仲裁信息
        List<ArbitrationDTO> list = new ArrayList<ArbitrationDTO>();
        for (MarkLibrary library : libraryList) {
            if (library.getStatus() != LibraryStatus.WAITING) {
                Marker marker = markerService.findById(library.getMarkerId());
                marker.setUser(userService.findById(marker.getUserId()));
                list.add(new ArbitrationDTO(library, marker));
            }
        }
        task.setArbitrationList(list);
        // 回评状态下的打分信息
        if (history.getTotalScore() != null) {
            task.setTotalScore(history.getTotalScore());
        }
        if (history.getScoreList() != null) {
            task.setScoreList(history.getScoreList());
        }
        return task;
    }

    @Override
    public Task build(MarkLibrary library) {
        ExamSubject subject = subjectService.find(library.getExamId(), library.getSubjectCode());
        ExamStudent student = studentService.findByExamIdAndExamNumber(library.getExamId(), library.getExamNumber());
        MarkGroup group = groupService.findOne(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());

        Task task = new Task();
        task.setExist(true);
        task.setStatusValue(MarkStatus.FORMAL.getValue());
        task.setStudentId(library.getStudentId());
        task.setLibraryId(library.getId());
        // 正评显示考生密号
        task.setStudentNumber(library.getSecretNumber());
        task.setMarkStepList(buildMarkStep(group, library.getId()));
        task.setPictureUrls(fileService.getSliceUris(library.getExamId(), library.getSecretNumber(), 1,
                student.getSliceCount()));
        task.setPictureConfig(group.getPictureConfigList());
        task.setJsonUrl(fileService.getJsonUri(library.getExamId(), library.getSecretNumber()));
        if (subject.getPaperFileType() != null) {
            task.setPaperUrl(fileService.getPaperUri(subject.getExamId(), subject.getCode(), subject.getPaperFileType()));
        }
        if (subject.getAnswerFileType() != null) {
            task.setAnswerUrl(fileService.getAnswerUri(subject.getExamId(), subject.getCode(),
                    subject.getAnswerFileType()));
        }
        task.setObjectiveScore(student.getObjectiveScore() != null ? student.getObjectiveScore() : 0);
        task.setMarkTime(library.getMarkerTime());
        task.setTagList(getMarkSpecialTagList(library.getId()));
        if (library.getMarkerScore() != null) {
            task.setTotalScore(library.getMarkerScore());
        }
        if (library.getMarkerScoreList() != null) {
            task.setScoreList(library.getMarkerScoreList());
        }
        // if (library.getStatus() == LibraryStatus.BACKED &&
        // library.getMarkerId() != null) {
        // task.setBack(true);
        // }
        return task;
    }

    @Override
    public Task build(TrialLibrary library, TrialHistory history) {
        ExamSubject subject = subjectService.find(library.getExamId(), library.getSubjectCode());
        ExamStudent student = studentService.findByExamIdAndExamNumber(library.getExamId(), library.getExamNumber());
        MarkGroup group = groupService.findOne(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());

        Task task = new Task();
        task.setExist(true);
        task.setStatusValue(MarkStatus.TRIAL.getValue());
        task.setStatusName(MarkStatus.TRIAL.getName());
        // 试评显示考生密号
        task.setStudentNumber(library.getSecretNumber());
        task.setStudentId(library.getStudentId());
        task.setLibraryId(library.getId());
        task.setMarkStepList(buildTrialStep(group, history));
        task.setPictureUrls(fileService.getSliceUris(library.getExamId(), library.getSecretNumber(), 1,
                student.getSliceCount()));
        task.setPictureConfig(group.getPictureConfigList());
        task.setJsonUrl(fileService.getJsonUri(library.getExamId(), library.getSecretNumber()));
        if (subject.getPaperFileType() != null) {
            task.setPaperUrl(fileService.getPaperUri(subject.getExamId(), subject.getCode(), subject.getPaperFileType()));
        }
        if (subject.getAnswerFileType() != null) {
            task.setAnswerUrl(fileService.getAnswerUri(subject.getExamId(), subject.getCode(),
                    subject.getAnswerFileType()));
        }
        task.setObjectiveScore(student.getObjectiveScore() != null ? student.getObjectiveScore() : 0);
        if (history != null) {
            task.setMarkTime(history.getMarkerTime());
            task.setTotalScore(history.getMarkerScore());
            task.setScoreList(history.getMarkerScoreList());
            task.setTagList(getTrialTagList(history));
        }
        return task;
    }

    private List<MarkStepDTO> buildMarkStep(MarkGroup group, Integer libraryId) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(group.getExamId(),
                group.getSubjectCode(), false, group.getNumber());
        int number = 0;
        for (ExamQuestion question : sList) {
            number++;

            MarkStepDTO step = buildStep(number, question);
            if (libraryId != null) {
                // 增加阅卷轨迹列表获取
                List<MarkTrack> tracks = trackService.findByLibraryId(libraryId);
                String questionNumber = question.getQuestionNumber();
                for (MarkTrack track : tracks) {
                    if (track.getQuestionNumber().equals(questionNumber)) {
                        step.addTrack(new TrackDTO(track));
                    }
                }
            }
            list.add(step);
        }
        return list;
    }

    private List<MarkStepDTO> buildTrialStep(MarkGroup group, TrialHistory history) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(group.getExamId(),
                group.getSubjectCode(), false, group.getNumber());
        int number = 0;
        for (ExamQuestion question : sList) {
            number++;

            MarkStepDTO step = buildStep(number, question);
            if (history != null) {
                // 增加阅卷轨迹列表获取
                List<TrialTrack> tracks = trialService.findTrack(history.getLibraryId(), history.getMarkerId());
                String questionNumber = question.getQuestionNumber();
                for (TrialTrack track : tracks) {
                    if (track.getQuestionNumber().equals(questionNumber)) {
                        step.addTrack(new TrackDTO(track));
                    }
                }
            }
            list.add(step);
        }
        return list;
    }

    private MarkStepDTO buildStep(int number, ExamQuestion question) {
        MarkStepDTO step = new MarkStepDTO();
        step.setQuestionNumber(question.getQuestionNumber());
        step.setMainNumber(question.getMainNumber());
        step.setSubNumber(question.getSubNumber());
        step.setNumber(number);
        step.setTitle(question.getMainTitle() + "-" + question.getSubNumber());
        step.setTotalScore(question.getTotalScore());
        step.setDefaultScore(0d);
        step.setMax(question.getTotalScore());
        step.setMin(0d);
        step.setInterval(question.getIntervalScore());
        step.setScoreList(question.getScoreListArray());
        return step;
    }

    public SpecialTagDTO[] getMarkSpecialTagList(Integer libraryId) {
        SpecialTagDTO[] specialTags = null;
        List<MarkSpecialTag> list = markSpecialTagService.findByLibraryId(libraryId);
        if (!list.isEmpty()) {
            specialTags = new SpecialTagDTO[list.size()];
            for (int i = 0; i < list.size(); i++) {
                SpecialTagDTO specialTagDTO = new SpecialTagDTO();
                specialTagDTO.setTagName(list.get(i).getTagName());
                specialTagDTO.setPositionX(list.get(i).getPositionX());
                specialTagDTO.setPositionY(list.get(i).getPositionY());
                specialTagDTO.setOffsetIndex(list.get(i).getOffsetIndex());
                specialTagDTO.setOffsetX(list.get(i).getOffsetX());
                specialTagDTO.setOffsetY(list.get(i).getOffsetY());
                specialTags[i] = specialTagDTO;
            }
        }
        return specialTags;
    }

    public SpecialTagDTO[] getTrialTagList(TrialHistory history) {
        SpecialTagDTO[] specialTags = null;
        List<TrialTag> list = trialService.findTag(history.getLibraryId(), history.getMarkerId());
        if (!list.isEmpty()) {
            specialTags = new SpecialTagDTO[list.size()];
            for (int i = 0; i < list.size(); i++) {
                SpecialTagDTO specialTagDTO = new SpecialTagDTO();
                specialTagDTO.setTagName(list.get(i).getContent());
                specialTagDTO.setPositionX(list.get(i).getPositionX());
                specialTagDTO.setPositionY(list.get(i).getPositionY());
                specialTagDTO.setOffsetIndex(list.get(i).getOffsetIndex());
                specialTagDTO.setOffsetX(list.get(i).getOffsetX());
                specialTagDTO.setOffsetY(list.get(i).getOffsetY());
                specialTags[i] = specialTagDTO;
            }
        }
        return specialTags;
    }

    @Override
    public Task build(Integer studentId) {
        ExamStudent student = studentService.findById(studentId);
        ExamSubject subject = subjectService.find(student.getExamId(), student.getSubjectCode());
        Task task = new Task();
        task.setExist(true);
        task.setStudentId(studentId);
        task.setStudentNumber(student.getSecretNumber());
        task.setMarkStepList(buildMarkStep(student));
        task.setPictureUrls(fileService.getSliceUris(student.getExamId(), student.getSecretNumber(), 1,
                student.getSliceCount()));
        task.setJsonUrl(fileService.getJsonUri(student.getExamId(), student.getSecretNumber()));
        if (subject.getPaperFileType() != null) {
            task.setPaperUrl(fileService.getPaperUri(subject.getExamId(), subject.getCode(), subject.getPaperFileType()));
        }
        if (subject.getAnswerFileType() != null) {
            task.setAnswerUrl(fileService.getAnswerUri(subject.getExamId(), subject.getCode(),
                    subject.getAnswerFileType()));
        }
        task.setObjectiveScore(student.getObjectiveScore());
        task.setBack(true);
        return task;
    }

    private List<PictureConfigItem> buildPictureConfig(int picCount) {
        List<PictureConfigItem> list = new LinkedList<PictureConfigItem>();
        for (int i = 1; i <= picCount; i++) {
            list.add(new PictureConfigItem(String.valueOf(i)));
        }
        return list;
    }

    private List<MarkStepDTO> buildMarkStep(ExamStudent student) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjective(student.getExamId(),
                student.getSubjectCode(), false);
        int number = 0;
        for (ExamQuestion question : sList) {
            number++;

            MarkStepDTO step = buildStep(number, question);
            // 增加阅卷轨迹列表获取
            List<MarkTrack> tracks = trackService.findByStudentId(student.getId());
            String questionNumber = question.getQuestionNumber();
            for (MarkTrack track : tracks) {
                if (track.getQuestionNumber().equals(questionNumber)) {
                    step.addTrack(new TrackDTO(track));
                }
            }
            list.add(step);
        }
        return list;
    }

    @Transactional
    @Override
    public boolean submitByStudent(Task task) {
        if (task == null) {
            return false;
        }
        try {
            List<MarkLibrary> libraries = libraryService.findByStudentId(task.getLibraryId());
            int start = 0;
            int end = 0;
            for (int i = 0; i < libraries.size(); i++) {
                MarkLibrary library = libraries.get(i);
                Task stepTask = new Task();
                stepTask.setProblem(task.isProblem());
                stepTask.setBack(task.isBack());
                stepTask.setLibraryId(library.getId());
                stepTask.setHeaderId(task.getHeaderId());
                List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(
                        library.getExamId(), library.getSubjectCode(), false, library.getGroupNumber());
                if (start == end) {
                    end = end + sList.size() * 2 - 1;
                } else {
                    start = end + 1;
                    end = start + sList.size() * 2 - 1;
                }
                String scoreList = task.getScoreList().substring(start, end);
                stepTask.setHeaderScoreList(scoreList);
                String[] scores = scoreList.split(",");
                double totalScore = 0;
                for (String score : scores) {
                    totalScore = totalScore + Double.parseDouble(score);
                }
                stepTask.setHeaderScore(totalScore);
                // submitTask(stepTask);
            }
            return true;
        } catch (Exception e) {
            log.error("task submit faile", e);
            return false;
        }
    }

}
