package cn.com.qmth.stmms.admin.exam;

import cn.com.qmth.stmms.biz.exam.model.CheckStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.query.CheckStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.CheckStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.enums.CheckType;
import cn.com.qmth.stmms.common.enums.LogType;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Controller
@RequestMapping("/admin/exam/check/student")
public class CheckStudentController extends BaseExamController {

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private CheckStudentService checkStudentService;

    @Autowired
    private FileService fileService;

    @Logging(menu = "人工确认查询", type = LogType.QUERY)
    @RequestMapping
    public String index(Model model, HttpServletRequest request, CheckStudentSearchQuery query) {
        int examId = getSessionExamId(request);
        List<ExamStudent> studentList = new ArrayList<ExamStudent>();
        query.setExamId(examId);
        query.setChecked(false);
        if (query.getType() == null) {
            query.setType(CheckType.MANUAL);
        }
        query = checkStudentService.findByQuery(query);
        for (CheckStudent c : query.getResult()) {
            studentList.add(studentService.findById(c.getStudentId()));
        }
        model.addAttribute("typeList", CheckType.values());
        model.addAttribute("studentList", studentList);
        model.addAttribute("query", query);
        return "modules/exam/checkStudent";
    }

    @RequestMapping("/start")
    @ResponseBody
    public ModelAndView start(HttpServletRequest request, @RequestParam CheckType type) {
        int examId = getSessionExamId(request);
        if (examId > 0) {
            ModelAndView view = new ModelAndView("modules/exam/checkStudentEdit");
            List<Integer> ids = new ArrayList<Integer>();
            List<CheckStudent> list = checkStudentService.findByExamIdAndIsCheckedAndType(examId, false, type);
            for (CheckStudent student : list) {
                ids.add(student.getStudentId());
            }
            view.addObject("fileServer", fileService.getFileServer());
            view.addObject("ids", StringUtils.join(ids, ","));
            return view;
        } else {
            return new ModelAndView("redirect:/admin/exam/list");
        }
    }

    @Logging(menu = "人工确认考生", type = LogType.UPDATE)
    @RequestMapping("/save")
    @ResponseBody
    public Object save(HttpServletRequest request, @RequestParam Integer studentId, @RequestParam String answers) {
        ExamStudent student = studentService.findById(studentId);
        CheckStudent cs = checkStudentService.findByStudentId(studentId);
        if (student != null && cs != null) {
            student.setAnswers(answers);
            cs.setChecked(true);
            cs.setUpdateTime(new Date());
            checkStudentService.save(cs);
            return true;
        } else {
            return false;
        }
    }

}
